@@ -12,7 +12,7 @@ Equal::Equal () : Infix ("==")
 {
 }
 
-double Equal::evaluateScalar(vector<double const *> const &args) const
+double Equal::evaluate(vector<double const *> const &args) const
 {
   return *args[0] == *args[1];
 }
