@@ -12,7 +12,7 @@ LessOrEqual::LessOrEqual () : Infix ("<=")
 {
 }
 
-double LessOrEqual::evaluateScalar(vector<double const *> const &args) const
+double LessOrEqual::evaluate(vector<double const *> const &args) const
 {
   return *args[0] <= *args[1];
 }
