@@ -12,7 +12,7 @@ LessThan::LessThan () : Infix ("<")
 {
 }
 
-double LessThan::evaluateScalar(vector<double const *> const &args) const
+double LessThan::evaluate(vector<double const *> const &args) const
 {
   return *args[0] < *args[1];
 }
