@@ -1,31 +1,37 @@
 #include <config.h>
+#include <sampler/GraphView.h>
 #include <graph/StochasticNode.h>
-#include <graph/NodeError.h>
 #include <distribution/Distribution.h>
-#include <sampler/DensitySampler.h>
 
 #include "RealSlicer.h"
 
 #include <cmath>
+#include <stdexcept>
 
 using std::vector;
 using std::string;
+using std::logic_error;
 
 namespace base {
 
-    RealSlicer::RealSlicer(double width, long maxwidth)
-	: Slicer(width, maxwidth)
+    RealSlicer::RealSlicer(GraphView const *gv, unsigned int chain,
+			   double width, long maxwidth)
+	: Slicer(width, maxwidth), _gv(gv), _chain(chain)
     {
-	
+	if (gv->nodes().size() != 1 || 
+	    !canSample(gv->nodes().front()))
+	{
+	    throw logic_error("Invalid RealSlicer");
+	}
     }
 
     bool 
     RealSlicer::canSample(StochasticNode const *node)
     {
-	if (node->distribution()->isDiscreteValued() || node->length() != 1)
+	if (node->isDiscreteValued() || node->length() != 1)
 	    return false;
 
-	if (df(node) == 0)
+	if (node->df() == 0)
 	    return false; 
 
 	return true;
@@ -33,18 +39,17 @@ namespace base {
 
     double RealSlicer::value() const
     {
-	return _sampler->nodes()[0]->value(_chain)[0];
+	return _gv->nodes().front()->value(_chain)[0];
     }
  
     void RealSlicer::setValue(double value)
     {
-	_sampler->setValue(&value, 1, _chain);
+	_gv->setValue(&value, 1, _chain);
     }
 
     void RealSlicer::getLimits(double *lower, double *upper) const
     {
-	StochasticNode const *snode = _sampler->nodes().front();
-	support(lower, upper, 1, snode, _chain);
+	_gv->nodes().front()->support(lower, upper, 1, _chain);
     }
 
     void RealSlicer::update(RNG *rng)
@@ -56,4 +61,9 @@ namespace base {
     {
 	return "RealSlicer";
     }
+
+    double RealSlicer::logDensity() const
+    {
+	return _gv->logFullConditional(_chain);
+    }
 }
