@@ -18,22 +18,22 @@ using std::max_element;
 using std::logic_error;
 
 #define PROB(par) (par[0])
-#define NCAT(dims) (dims[0][0])
+#define NCAT(lengths) (lengths[0])
 
 DCat::DCat()
-    : Distribution("dcat", 1, false, true) 
+    : VectorDist("dcat", 1) 
 {}
 
-bool DCat::checkParameterDim(vector<vector<unsigned int> > const &dims) const
+bool DCat::isDiscreteValued(vector<bool> const &mask) const
 {
-    return isVector(dims[0]) || isScalar(dims[0]);
+    return true;
 }
 
 bool DCat::checkParameterValue(vector<double const *> const &par,
-			       vector<vector<unsigned int> > const &dims) const
+			       vector<unsigned int> const &lengths) const
 {
     bool nz = false;
-    for (unsigned int i = 0; i < NCAT(dims); i++) {
+    for (unsigned int i = 0; i < NCAT(lengths); i++) {
 	if (PROB(par)[i] < 0.0) {
 	    return false; 
 	}
@@ -46,16 +46,16 @@ bool DCat::checkParameterValue(vector<double const *> const &par,
 
 double DCat::logLikelihood(double const *x, unsigned int length,
 			   vector<double const *> const &par,
-			   vector<vector<unsigned int> > const &dims,
+			   vector<unsigned int> const &lengths,
 			   double const *lower, double const *upper) const
 {
     unsigned int y = static_cast<unsigned int>(*x);
-    if (y < 1 || y > NCAT(dims)) {
+    if (y < 1 || y > NCAT(lengths)) {
 	return JAGS_NEGINF;
     }
     else {
 	double sump = 0.0;
-	for (unsigned int i = 0; i < NCAT(dims); i++) {
+	for (unsigned int i = 0; i < NCAT(lengths); i++) {
 	    sump += PROB(par)[i];
 	}
 	return log(PROB(par)[y-1]) - log(sump);
@@ -64,14 +64,14 @@ double DCat::logLikelihood(double const *x, unsigned int length,
 
 void DCat::randomSample(double *x, unsigned int length,
 			vector<double const *> const &par,
-			vector<vector<unsigned int> > const &dims,
+			vector<unsigned int> const &lengths,
 			double const *lower, double const *upper,
 			RNG *rng) const
 {
     double sump = 0;
     unsigned int i = 0;
 
-    for ( ; i < NCAT(dims); ++i) {
+    for ( ; i < NCAT(lengths); ++i) {
 	sump += PROB(par)[i];
     }
     double p = sump * rng->uniform();    
@@ -86,24 +86,34 @@ void DCat::randomSample(double *x, unsigned int length,
 
 void DCat::support(double *lower, double *upper, unsigned int length,
 	           vector<double const *> const &par,
-	           vector<vector<unsigned int> > const &dims) const
+	           vector<unsigned int> const &lengths) const
 {
     if (length != 1)
 	throw logic_error("Invalid length in DCat::support");
 
     *lower = 1;
-    *upper = NCAT(dims);
+    *upper = NCAT(lengths);
 }
 
 void DCat::typicalValue(double *x, unsigned int length,
 			vector<double const *> const &par,
-			vector<std::vector<unsigned int> > const &dims,
+			vector<unsigned int> const &lengths,
 			double const *lower, double const *upper) const
 {
-    *x = max_element(PROB(par), PROB(par) + NCAT(dims)) - PROB(par) + 1;
+    *x = max_element(PROB(par), PROB(par) + NCAT(lengths)) - PROB(par) + 1;
 }
 
-bool DCat::isSupportFixed(std::vector<bool> const &fixmask) const
+bool DCat::isSupportFixed(vector<bool> const &fixmask) const
 {
     return true;
 }
+
+bool DCat::checkParameterLength(vector<unsigned int> const &lengths) const
+{
+    return true;
+}
+
+unsigned int DCat::length(vector<unsigned int> const &lengths) const
+{
+    return 1;
+}
