@@ -22,9 +22,14 @@ using std::min;
 using std::logic_error;
 
 DHyper::DHyper()
-    : DistScalarRmath("dhyper", 4, DIST_SPECIAL, false, true)
+    : RScalarDist("dhyper", 4, DIST_SPECIAL, true)
 {}
 
+bool DHyper::canBound() const
+{
+    return false;
+}
+
 static void
 getParameters(int &n1, int &n2, int &m1, double &psi,
 	      vector<double const *> const &parameters)
@@ -47,13 +52,11 @@ DHyper::checkParameterDiscrete (vector<bool> const &mask) const
   return true;
 }
 
-bool DHyper::checkParameterValue (vector<double const *> const &parameters,
-				  vector<vector<unsigned int> > const &dims) 
-  const
+bool DHyper::checkParameterValue (vector<double const *> const &params) const
 {
   int n1,n2,m1;
   double psi;
-  getParameters(n1, n2, m1, psi, parameters);
+  getParameters(n1, n2, m1, psi, params);
 
   if (n1 < 0 || n2 < 0)
     return false;
