@@ -1,7 +1,7 @@
 #ifndef DPOIS_H_
 #define DPOIS_H_
 
-#include <distribution/DistScalarRmath.h>
+#include "RScalarDist.h"
 
 /**
  * <pre>
@@ -10,11 +10,12 @@
  * </pre>
  * @short Poisson distribution
  */
-class DPois : public DistScalarRmath {
+class DPois : public RScalarDist {
  public:
   DPois();
 
-  double d(double x, std::vector<double const *> const &parameters, bool give_log) const;
+  double d(double x, std::vector<double const *> const &parameters, 
+	   bool give_log) const;
   double p(double q, std::vector<double const *> const &parameters, bool lower,
 	   bool give_log) const;
   double q(double p, std::vector<double const *> const &parameters, bool lower,
@@ -23,9 +24,7 @@ class DPois : public DistScalarRmath {
   /**
    * Checks that lambda > 0
    */
-  bool checkParameterValue(std::vector<double const *> const &parameters,
-			   std::vector<std::vector<unsigned int> > const &dims)
-    const;
+  bool checkParameterValue(std::vector<double const *> const &parameters) const;
 };
 
 #endif /* DPOIS_H_ */
