@@ -6,13 +6,12 @@ using std::vector;
 
 namespace bugs {
 
-    InProd::InProd () : Function ("inprod", 2)
+    InProd::InProd () : VectorFunction ("inprod", 2)
     {
     }
 
     void InProd::evaluate(double *x, vector<double const *> const &args,
-			  vector<unsigned int> const &lengths,
-			  vector<vector<unsigned int> > const &dims) const
+			  vector<unsigned int> const &lengths) const
     {
 	double svalue = 0;
 	for (unsigned int i = 0; i < lengths[0]; i++)
@@ -23,9 +22,9 @@ namespace bugs {
     }
 
     bool 
-    InProd::checkParameterDim (vector<vector<unsigned int> > const &dims) const
+    InProd::checkParameterLength (vector<unsigned int> const &lengths) const
     {
-	return (dims[0] == dims[1]);
+	return (lengths[0] == lengths[1]);
     }
 
     bool InProd::isDiscreteValued(std::vector<bool> const &mask) const
@@ -34,32 +33,17 @@ namespace bugs {
     }
 
     bool 
-    InProd::isLinear(vector<bool> const &mask, vector<bool> const &fix) const
+    InProd::isScale(vector<bool> const &mask, vector<bool> const &fix) const
     {
 	//Test for quadratic term
 	if (mask[0] && mask[1])
 	    return false;
 
-	if (!fix.empty()) {
-	    if ( (!mask[0] && !fix[0]) || (!mask[1] && !fix[1]) )
-		return false;
-	}
-
-	return true;
-    }
-
-    bool InProd::isScale(unsigned int index, vector<bool> const &fix) const
-    {
 	if (fix.empty()) {
 	    return true;
 	}
-	else {
-	    if ((index == 0 && !fix[1]) || (index == 1 && !fix[0])) {
-		return false;
-	    }
-	}
-
-	return true;
+        else {
+	    return (mask[0] || fix[0]) && (mask[1] || fix[1]); 
+        }
     }
-
 }
