@@ -7,12 +7,11 @@ using std::vector;
 
 namespace bugs {
 
-    Inverse::Inverse ():Function ("inverse", 1)
+    Inverse::Inverse (): ArrayFunction ("inverse", 1)
     {
     }
 
     void Inverse::evaluate (double *value, vector<double const *> const &args,
-			    vector<unsigned int> const &lengths,
 			    vector<vector<unsigned int> > const &dims) const
     {
 	inverse (value, args[0], dims[0][0], false);
