@@ -3,17 +3,13 @@
 
 #include "ConjugateMethod.h"
 
-class StochasticNode;
-class Graph;
-
 class ConjugateGamma : public ConjugateMethod {
     double *_coef;
 public:
-    ConjugateGamma();
+    ConjugateGamma(GraphView const *gv);
     ~ConjugateGamma();
     static bool canSample(StochasticNode *snode, Graph const &graph);
-    void update(ConjugateSampler *sampler, unsigned int chain, RNG *rng) const;
-    void initialize(ConjugateSampler *sampler, Graph const &graph);
+    void update(unsigned int chain, RNG *rng) const;
     std::string name() const;
 };
 
