@@ -4,7 +4,6 @@
 #include "Dirichlet.h"
 
 #include <sampler/SingletonFactory.h>
-class StochasticNode;
 
 /**
  * @short Factory object for multivariate normal samplers
@@ -13,8 +12,8 @@ class DirichletFactory : public SingletonFactory
 {
 public:
     bool canSample(StochasticNode *snode, Graph const &graph) const;
-    Sampler *makeSingletonSampler(StochasticNode *snode, Graph const &graph)
-                                  const;
+    Sampler *makeSampler(StochasticNode *snode, Graph const &graph) const;
+    std::string name() const;
 };
 
 #endif /* DIRICHLET_FACTORY_H_ */
